/**
* oon2.cpp - Example program for the PeLib library.
* OON2 (The 2nd version of "Out of nowhere") creates a valid PE file
* from scratch. The creation of the header structures are automized
* as much as possible. The file produced by oon2 ("dummy.exe")
* shows a messagebox when you run it.
*
* Copyright (c) 2004 Sebastian Porst (webmaster@the-interweb.com)
* All rights reserved.
*
* This software is licensed under the zlib/libpng License.
* For more details see http://www.opensource.org/licenses/zlib-license.php
* or the license information file (license.htm) in the root directory 
* of PeLib.
**/

#include "../../source/PeLib.h"
#include <iostream>
#include <string>
#include <vector>

#define ROUND_UP(N, S) ((((N) + (S) - 1) / (S)) * (S))

int main()
{
	std::string strFilename = "dummy.exe";

    // Truncate existing file	
    {
        std::ofstream a(strFilename, std::ios::trunc);
    }

	PeLib::PeFile32 f(strFilename);

	unsigned int uiSizeMzHeader = f.mzHeader().size(), LargeGap = 0x1001;

	// MZ header.
    {
	    std::cout << "Building MZ header..." << std::endl;
	    f.mzHeader().setAddressOfPeHeader(uiSizeMzHeader);
	    f.mzHeader().makeValid();
	    f.mzHeader().write(strFilename, 0);

	    // Then we need a valid PE header.
	    std::cout << "Building PE header..." << std::endl;
	    f.peHeader().setAddressOfEntryPoint(0x1000);
	    f.peHeader().makeValid(uiSizeMzHeader);
    }

    // Add imports
    {
        f.impDir().addFunction("User32.dll", "MessageBoxA");
        f.impDir().addFunction("User32.dll", "TranslateAcceleratorW");
        // Addresses will be set later
        f.iatDir().addAddress(0);
        f.iatDir().addAddress(0);
    }

    // Add sections
    {
	    f.peHeader().addSection(".code", 0x200);
	    f.peHeader().addSection(".rdata", ROUND_UP(f.iatDir().size() + LargeGap + f.impDir().size(), 0x200));

        f.peHeader().setCharacteristics(0, 0x60000020);
        f.peHeader().setCharacteristics(1, 0x40000040);
	}

    // IMport crap
	{
        for(unsigned int i = 0; i < 16; i++) f.peHeader().addDataDirectory();

	    unsigned uiIatDir = f.peHeader().getVirtualAddress(1), uiImpDir = uiIatDir + f.iatDir().size() + LargeGap;
	    f.impDir().write(strFilename, f.peHeader().rvaToOffset(uiImpDir), uiImpDir);
        f.iatDir().write(strFilename, f.peHeader().rvaToOffset(uiIatDir));
	    f.peHeader().setIddImportRva(uiImpDir);
	    f.peHeader().setIddImportSize(f.impDir().size());
        f.peHeader().setIddIatRva(uiIatDir);
        f.peHeader().setIddIatSize(f.iatDir().size());
    }

    // 
    {	
	    f.peHeader().setImageBase(0x01000000);
	    f.peHeader().setMajorSubsystemVersion(4);
	    f.peHeader().setSubsystem(2);
	    f.peHeader().makeValid(uiSizeMzHeader);

        f.peHeader().setBaseOfData(f.peHeader().getVirtualAddress(1));
        f.peHeader().setSizeOfInitializedData(f.peHeader().getVirtualSize(1));

	    f.peHeader().write(strFilename, uiSizeMzHeader);

	    // Write the new section to the output file.
	    f.peHeader().writeSections(f.getFileName());
    }

	// Write program code to file.
    {
        char data[] = {0xC3};
	    std::cout << "Writing code" << std::endl;
	    std::vector<PeLib::byte> vData(data, data + sizeof(data)/sizeof(data[0]));
	    f.peHeader().writeSectionData(f.getFileName(), 0, vData);
    }

    {
        std::cout << f.readImportDirectory() << std::endl;
        for(unsigned int i = 0; i < f.iatDir().calcNumberOfAddresses(); i++)
        {
            f.iatDir().setAddress(i, f.impDir().getFirstThunk(0, i, PeLib::OLDDIR));
        }

        unsigned uiIatDir = f.peHeader().getVirtualAddress(1);
        f.iatDir().write(f.getFileName(), f.peHeader().rvaToOffset(uiIatDir));
    }
}
